@@ -25,13 +25,13 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <getopt.h>
+#include "my_getopt.h"
 #ifdef HAVE_VALGRIND_VALGRIND_H
 #  include <valgrind/valgrind.h>
 #endif
 
-#include <dwg.h>
 #include "common.h"
+#include <dwg.h>
 #include "suffix.inc"
 
 static int opts = 0;
@@ -52,9 +52,9 @@ opt_version (void)
 static int
 help (void)
 {
-  printf ("\nUsage: dwgbmp [OPTION]... DWGFILE [BMPFILE]\n");
-  printf ("Extract the DWG thumbnail image as BMP.\n");
-  printf ("Default BMPFILE: DWGFILE with .bmp extension.\n"
+  printf ("\nUsage: dwgbmp [OPTION]... DWGFILE [thumbnailfile]\n");
+  printf ("Extract the DWG thumbnail image as BMP, WMF or PNG.\n");
+  printf ("Default thumbnailfile: DWGFILE with the proper extension.\n"
           "\n");
 #ifdef HAVE_GETOPT_LONG
   printf ("  -v[0-9], --verbose [0-9]  verbosity\n");
@@ -77,7 +77,7 @@ bmp_free_dwg (Dwg_Data *dwg)
 {
 #if defined __SANITIZE_ADDRESS__ || __has_feature(address_sanitizer)
   {
-    char *asanenv = getenv("ASAN_OPTIONS");
+    char *asanenv = getenv ("ASAN_OPTIONS");
     if (!asanenv)
       force_free = 1;
     // detect_leaks is enabled by default. see if it's turned off
@@ -86,8 +86,7 @@ bmp_free_dwg (Dwg_Data *dwg)
   }
 #endif
   // really huge DWG's need endlessly here.
-  if ((dwg->header.version && dwg->num_objects < 1000)
-      || force_free
+  if ((dwg->header.version && dwg->num_objects < 1000) || force_free
 #ifdef HAVE_VALGRIND_VALGRIND_H
       || (RUNNING_ON_VALGRIND)
 #endif
@@ -98,11 +97,12 @@ bmp_free_dwg (Dwg_Data *dwg)
 #pragma pack(1)
 
 static int
-get_bmp (char *dwgfile, char *bmpfile)
+get_bmp (char *dwgfile, char *bmpfile, bool must_free)
 {
   unsigned char *data;
   int error;
   BITCODE_RL size;
+  BITCODE_RC type = 0;
   size_t retval;
   FILE *fh;
   Dwg_Data dwg;
@@ -113,6 +113,8 @@ get_bmp (char *dwgfile, char *bmpfile)
     BITCODE_RL reserved;
     BITCODE_RL offset;
   } bmp_h;
+  const char *typenames[] = { "", "header", "bmp", "wmf", "", "", "png" };
+  const char *typename;
 
   memset (&dwg, 0, sizeof (Dwg_Data));
   dwg.opts = opts;
@@ -121,21 +123,37 @@ get_bmp (char *dwgfile, char *bmpfile)
   if (error >= DWG_ERR_CRITICAL)
     {
       fprintf (stderr, "Unable to read file %s. ERROR 0x%x\n", dwgfile, error);
+      if (must_free)
+        free (bmpfile);
       bmp_free_dwg (&dwg);
       return error;
     }
 
   /* Get DIB bitmap data */
-  data = dwg_bmp (&dwg, &size);
+  data = dwg_bmp (&dwg, &size, &type);
+  if (type <= 6)
+    typename = typenames[type];
+  else
+    typename = "";
   if (!data)
     {
-      fprintf (stderr, "No thumbnail bmp image in %s\n", dwgfile);
+      fprintf (stderr, "No thumbnail image in %s\n", dwgfile);
+      if (must_free)
+        free (bmpfile);
+      bmp_free_dwg (&dwg);
+      return 0;
+    }
+  if (!*typename)
+    {
+      fprintf (stderr, "Unknown thumbnail type %u in %s\n", type, dwgfile);
       bmp_free_dwg (&dwg);
       return 0;
     }
   if (size < 1)
     {
       fprintf (stderr, "Empty thumbnail data in %s\n", dwgfile);
+      if (must_free)
+        free (bmpfile);
       bmp_free_dwg (&dwg);
       return -3;
     }
@@ -143,54 +161,75 @@ get_bmp (char *dwgfile, char *bmpfile)
     {
       fprintf (stderr,
                "Invalid thumbnail data in %s,"
-               " size " FORMAT_RL " > %lu\n",
+               " size " FORMAT_RL " > %" PRIuSIZE "\n",
                dwgfile, size, dwg.thumbnail.size);
+      if (must_free)
+        free (bmpfile);
       bmp_free_dwg (&dwg);
       return -3;
     }
+  if (type != 2 && *typename)
+    {
+      if (must_free)
+        free (bmpfile);
+      bmpfile = suffix (dwgfile, typename);
+    }
 
   fh = fopen (bmpfile, "w");
   if (!fh)
     {
-      fprintf (stderr, "Unable to write BMP file '%s'\n", bmpfile);
+      fprintf (stderr, "Unable to write thumbnail file '%s'\n", bmpfile);
+      if (must_free)
+        free (bmpfile);
       bmp_free_dwg (&dwg);
       return -4;
     }
 
-  /* Write bmp file header */
-  bmp_h.magic[0] = 'B';
-  bmp_h.magic[1] = 'M';
-  bmp_h.file_size = 14 + size; // file header + DIB data
-  bmp_h.reserved = 0;
-  bmp_h.offset = 14 + 40 + 4 * 256; // file header + DIB header + color table
-  retval = fwrite (&bmp_h.magic[0], sizeof (char), 2, fh);
-  if (!retval)
+  if (type == 2)
     {
-      bmp_free_dwg (&dwg);
-      perror ("writing BMP magic");
-      return 1;
-    }
-  retval = fwrite (&bmp_h.file_size, 4, 3, fh);
-  if (!retval)
-    {
-      bmp_free_dwg (&dwg);
-      perror ("writing BMP file_size");
-      return 1;
+      /* Write bmp file header */
+      bmp_h.magic[0] = 'B';
+      bmp_h.magic[1] = 'M';
+      bmp_h.file_size = 14 + size; // file header + DIB data
+      bmp_h.reserved = 0;
+      bmp_h.offset
+          = 14 + 40 + 4 * 256; // file header + DIB header + color table
+      retval = fwrite (&bmp_h.magic[0], sizeof (char), 2, fh);
+      if (!retval)
+        {
+          bmp_free_dwg (&dwg);
+          perror ("writing BMP magic");
+          fclose (fh);
+          return 1;
+        }
+      retval = fwrite (&bmp_h.file_size, 4, 3, fh);
+      if (!retval)
+        {
+          if (must_free)
+            free (bmpfile);
+          bmp_free_dwg (&dwg);
+          perror ("writing BMP file_size");
+          fclose (fh);
+          return 1;
+        }
     }
 
-  /* Write data (DIB header + bitmap) */
+  /* Write data: DIB header + bitmap, resp. the others */
   retval = fwrite (data, sizeof (char), size, fh);
+  fclose (fh);
   if (!retval)
     {
+      if (must_free)
+        free (bmpfile);
       bmp_free_dwg (&dwg);
-      perror ("writing BMP header");
+      perror ("writing thumbnail data");
       return 1;
     }
-  fclose (fh);
 
   printf ("Success. Written thumbnail image to '%s'\n", bmpfile);
   bmp_free_dwg (&dwg);
-
+  if (must_free)
+    free (bmpfile);
   return 0;
 }
 
@@ -200,6 +239,7 @@ main (int argc, char *argv[])
   int i = 1, error;
   char *dwgfile, *bmpfile;
   int c;
+  bool must_free;
 #ifdef HAVE_GETOPT_LONG
   int option_index = 0;
   static struct option long_options[]
@@ -298,8 +338,6 @@ main (int argc, char *argv[])
     bmpfile = argv[i + 1];
   else
     bmpfile = suffix (dwgfile, "bmp");
-  error = get_bmp (dwgfile, bmpfile);
-  if (i != argc - 2)
-    free (bmpfile);
-  return error;
+  must_free = i != argc - 2;
+  return get_bmp (dwgfile, bmpfile, must_free);
 }
