@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2009-2020 Free Software Foundation, Inc.                   */
+/*  Copyright (C) 2009-2023 Free Software Foundation, Inc.                   */
 /*  Copyright (C) 2010 Thien-Thi Nguyen                                      */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
@@ -29,7 +29,7 @@
 #ifdef AX_STRCASECMP_HEADER
 #  include AX_STRCASECMP_HEADER
 #endif
-#include <getopt.h>
+#include "my_getopt.h"
 #ifdef HAVE_VALGRIND_VALGRIND_H
 #  include <valgrind/valgrind.h>
 #endif
@@ -44,6 +44,10 @@
 #  endif
 #endif
 
+#if !defined HAVE_STRCASECMP
+int strcasecmp (const char *a, const char *b);
+#endif
+
 static int opts = 1;
 
 static int
@@ -72,30 +76,31 @@ help (void)
           "\n");
 #ifdef HAVE_GETOPT_LONG
   printf ("  -v[0-9], --verbose [0-9]  verbosity\n");
-#ifndef DISABLE_DXF
-#  ifndef DISABLE_JSON
+#  ifndef DISABLE_DXF
+#    ifndef DISABLE_JSON
   printf ("  -O fmt,  --format fmt     fmt: DXF, DXFB, JSON, GeoJSON\n");
-#  else
+#    else
   printf ("  -O fmt,  --format fmt     fmt: DXF, DXFB\n");
-#  endif
+#    endif
   printf ("           Planned output formats:  YAML, XML/OGR, GPX, SVG, PS\n");
   printf ("  -o outfile                also defines the output fmt. Default: "
           "stdout\n");
-#endif
+#  endif
   printf ("           --help           display this help and exit\n");
   printf ("           --version        output version information and exit\n"
           "\n");
 #else
   printf ("  -v[0-9]     verbosity\n");
-#ifndef DISABLE_DXF
-#  ifndef DISABLE_JSON
+#  ifndef DISABLE_DXF
+#    ifndef DISABLE_JSON
   printf ("  -O fmt      fmt: DXF, DXFB, JSON, GeoJSON\n");
-#  else
+#    else
   printf ("  -O fmt      fmt: DXF, DXFB\n");
-#  endif
-  printf ("              Planned output formats:  YAML, XML/OGR, GPX, SVG, PS\n");
+#    endif
+  printf (
+      "              Planned output formats:  YAML, XML/OGR, GPX, SVG, PS\n");
   printf ("  -o outfile  also defines the output fmt. Default: stdout\n");
-#endif
+#  endif
   printf ("  -h          display this help and exit\n");
   printf ("  -i          output version information and exit\n"
           "\n");
@@ -120,23 +125,21 @@ main (int argc, char *argv[])
   int option_index = 0;
   static struct option long_options[]
       = { { "verbose", 1, &opts, 1 }, // optional
-          { "format", 1, 0, 'O' },    { "file", 1, 0, 'o' },
-          { "help", 0, 0, 0 },        { "version", 0, 0, 0 },
-          { "force-free", 0, 0, 0 },
-          { NULL, 0, NULL, 0 } };
+          { "format", 1, NULL, 'O' },   { "file", 1, NULL, 'o' },
+          { "help", 0, NULL, 0 },       { "version", 0, NULL, 0 },
+          { "force-free", 0, NULL, 0 }, { NULL, 0, NULL, 0 } };
 #endif
 
   if (argc < 2)
     return usage ();
 
-  while
-#ifdef HAVE_GETOPT_LONG
-      ((c = getopt_long (argc, argv, ":v::O:o:h", long_options, &option_index))
-       != -1)
-#else
-      ((c = getopt (argc, argv, ":v::O:o:hi")) != -1)
-#endif
+  while (1)
     {
+#ifdef HAVE_GETOPT_LONG
+      c = getopt_long (argc, argv, "v::O:o:h", long_options, &option_index);
+#else
+      c = getopt (argc, argv, "v::O:o:hi");
+#endif
       if (c == -1)
         break;
       switch (c)
@@ -181,27 +184,25 @@ main (int argc, char *argv[])
           return opt_version ();
 #endif
         case 'O':
-          fmt = optarg;
+          fmt = strdup (optarg);
           break;
         case 'o':
-          outfile = optarg;
+          outfile = strdup (optarg);
           if (!fmt && outfile != NULL)
             {
 #ifndef DISABLE_DXF
-#ifndef DISABLE_JSON
-              if (strstr (outfile, ".json") || strstr (outfile, ".JSON"))
-                fmt = (char *)"json";
+              if (strstr (outfile, ".dxf") || strstr (outfile, ".DXF"))
+                fmt = strdup ("dxf");
+              else if (strstr (outfile, ".dxfb") || strstr (outfile, ".DXFB"))
+                fmt = strdup ("dxfb");
               else
 #endif
-              if (strstr (outfile, ".dxf") || strstr (outfile, ".DXF"))
-                fmt = (char *)"dxf";
-              else if (strstr (outfile, ".dxfb") || strstr (outfile, ".DXFB"))
-                fmt = (char *)"dxfb";
 #ifndef DISABLE_JSON
+                  if (strstr (outfile, ".json") || strstr (outfile, ".JSON"))
+                fmt = strdup ("json");
               else if (strstr (outfile, ".geojson")
                        || strstr (outfile, ".GeoJSON"))
-                fmt = (char *)"geojson";
-#endif
+                fmt = strdup ("geojson");
               else
 #endif
                 fprintf (stderr, "Unknown output format for %s\n", outfile);
@@ -270,10 +271,11 @@ main (int argc, char *argv[])
         dat.fh = stdout;
       fprintf (stderr, "\n");
       dat.version = dat.from_version = dwg.header.version;
+      dat.codepage = dwg.header.codepage;
       // TODO --as-rNNNN version? for now not.
       // we want the native dump, converters are separate.
 #ifndef DISABLE_DXF
-#ifndef DISABLE_JSON
+#  ifndef DISABLE_JSON
       if (!strcasecmp (fmt, "json"))
         {
           if (opts > 1 && outfile)
@@ -281,8 +283,8 @@ main (int argc, char *argv[])
           error = dwg_write_json (&dat, &dwg);
         }
       else
-#endif
-      if (!strcasecmp (fmt, "dxfb"))
+#  endif
+          if (!strcasecmp (fmt, "dxfb"))
         {
           if (opts > 1 && outfile)
             fprintf (stderr, "Writing Binary DXF file %s\n", outfile);
@@ -294,7 +296,7 @@ main (int argc, char *argv[])
             fprintf (stderr, "Writing Binary DXF file %s\n", outfile);
           error = dwg_write_dxf (&dat, &dwg);
         }
-#ifndef DISABLE_JSON
+#  ifndef DISABLE_JSON
       else if (!strcasecmp (fmt, "geojson"))
         {
           if (opts > 1 && outfile)
@@ -302,7 +304,7 @@ main (int argc, char *argv[])
           error = dwg_write_geojson (&dat, &dwg);
         }
       else
-#endif
+#  endif
 #endif
         fprintf (stderr, "Invalid output format '%s'\n", fmt);
 
@@ -310,10 +312,10 @@ main (int argc, char *argv[])
         fclose (dat.fh);
     }
 
- done:
+done:
 #if defined __SANITIZE_ADDRESS__ || __has_feature(address_sanitizer)
   {
-    char *asanenv = getenv("ASAN_OPTIONS");
+    char *asanenv = getenv ("ASAN_OPTIONS");
     if (!asanenv)
       force_free = 1;
     // detect_leaks is enabled by default. see if it's turned off
@@ -323,13 +325,16 @@ main (int argc, char *argv[])
 #endif
 
   // forget about valgrind. really huge DWG's need endlessly here.
-  if ((dwg.header.version && dwg.num_objects < 1000)
-      || force_free
+  if ((dwg.header.version && dwg.num_objects < 1000) || force_free
 #ifdef HAVE_VALGRIND_VALGRIND_H
       || (RUNNING_ON_VALGRIND)
 #endif
   )
     {
+      if (fmt)
+        free ((char *)fmt);
+      if (outfile)
+        free ((char *)outfile);
       dwg_free (&dwg);
     }
 
