@@ -21,9 +21,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <getopt.h>
+#ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+#endif
+#include "my_stat.h"
+#include "my_getopt.h"
 #ifdef HAVE_VALGRIND_VALGRIND_H
 #  include <valgrind/valgrind.h>
 #endif
@@ -34,6 +36,12 @@
 #include "logging.h"
 #include "suffix.inc"
 
+#ifdef __AFL_COMPILER
+#  include "decode.h"
+#  include "encode.h"
+#  include "in_dxf.h"
+#endif
+
 static int help (void);
 
 static int opts = 1;
@@ -64,8 +72,7 @@ help (void)
   printf ("Default DWGFILE: DXFFILE with .dwg extension in the current "
           "directory.\n"
           "Existing files are not overwritten, unless -y is given.\n"
-          "Encoding currently only works for R13-R2000, dxf2dwg is highly "
-          "experimental.\n"
+          "Encoding currently only works for R13-R2000.\n"
           "\n");
 #ifdef HAVE_GETOPT_LONG
   printf ("  -v[0-9], --verbose [0-9]  verbosity\n");
@@ -106,6 +113,67 @@ __asan_default_options (void)
 }
 #endif
 
+#ifdef __AFL_COMPILER
+__AFL_FUZZ_INIT ();
+// fastest mode via shared mem
+#  define AFL_SHARED_MEM
+
+int
+main (int argc, char *argv[])
+{
+  Dwg_Data dwg;
+  Bit_Chain dat = { NULL, 0, 0, 0, 0 };
+  Bit_Chain out_dat = { NULL, 0, 0, 0, 0 };
+  FILE *fp;
+  struct stat attrib;
+
+  __AFL_INIT ();
+  // dat.opts = 3;
+#  ifdef AFL_SHARED_MEM
+  dat.chain = __AFL_FUZZ_TESTCASE_BUF;
+#  endif
+
+  while (__AFL_LOOP (10000))
+    { // llvm_mode persistent, non-forking mode
+#  ifdef AFL_SHARED_MEM
+      dat.size = __AFL_FUZZ_TESTCASE_LEN;
+      printf ("Fuzzing in_dxf + encode from shmem (%lu)\n", dat.size);
+#  elif 0 // still 10x faster than the old file-forking fuzzer.
+      /* from stdin: */
+      dat.size = 0;
+      // dat.chain = NULL;
+      dat_read_stream (&dat, stdin);
+      printf ("Fuzzing in_dxf + encode from stdin (%lu)\n", dat.size);
+#  else
+      /* else from file */
+      fp = fopen (argv[1], "rb");
+      if (!fp)
+        return 0;
+      dat.size = 0;
+      dat_read_file (&dat, fp, argv[1]);
+      fclose (fp);
+      printf ("Fuzzing in_dxf + encode from file (%lu)\n", dat.size);
+#  endif
+
+      if (dat.size < 100)
+        continue; // useful minimum input length
+      if (dwg_read_dxf (&dat, &dwg) <= DWG_ERR_CRITICAL)
+        {
+          memset (&out_dat, 0, sizeof (out_dat));
+          bit_chain_set_version (&out_dat, &dat);
+          dat.codepage = dwg.header.codepage;
+          out_dat.version = R_2000;
+          dwg_encode (&dwg, &out_dat);
+          free (out_dat.chain);
+          dwg_free (&dwg);
+        }
+    }
+  dwg_free (&dwg);
+}
+#  define main orig_main
+int orig_main (int argc, char *argv[]);
+#endif
+
 int
 main (int argc, char *argv[])
 {
@@ -125,8 +193,8 @@ main (int argc, char *argv[])
       = { { "verbose", 1, &opts, 1 }, // optional
           { "file", 1, 0, 'o' },      { "as", 1, 0, 'a' },
           { "overwrite", 0, 0, 'y' }, { "help", 0, 0, 0 },
-          { "force-free", 0, 0, 0 },
-          { "version", 0, 0, 0 },     { NULL, 0, NULL, 0 } };
+          { "force-free", 0, 0, 0 },  { "version", 0, 0, 0 },
+          { NULL, 0, NULL, 0 } };
 #endif
 
   if (argc < 2)
@@ -250,12 +318,20 @@ main (int argc, char *argv[])
         }
 
       dwg.opts = opts;
-      printf ("Warning: dxf2dwg is still highly experimental.\n");
+      dwg.header.version = dwg_version;
       printf ("Reading DXF file %s\n", filename_in);
       error = dxf_read_file (filename_in, &dwg);
       if (error >= DWG_ERR_CRITICAL)
         {
           fprintf (stderr, "READ ERROR 0x%x %s\n", error, filename_in);
+          if (need_free)
+            free (filename_out);
+          if (do_free
+#ifdef HAVE_VALGRIND_VALGRIND_H
+              || (RUNNING_ON_VALGRIND)
+#endif
+              )
+            dwg_free (&dwg);
           continue;
         }
 
@@ -272,10 +348,14 @@ main (int argc, char *argv[])
         }
       else
         {
-          // FIXME: for now only R_2000. later remove this line.
-          dwg.header.version = dwg_version;
+          // FIXME: for now only R_13b1 - R_2000. later remove this line.
+          if (dwg.header.from_version < R_13b1
+              || dwg.header.from_version >= R_2004)
+            dwg.header.version = dwg_version;
           if (dwg.header.from_version == R_INVALID)
             dwg.header.from_version = dwg.header.version;
+          if (dwg.header.version == R_INVALID)
+            dwg.header.version = dwg.header.from_version;
           printf ("\n");
         }
 
@@ -305,12 +385,12 @@ main (int argc, char *argv[])
                     error = dwg_write_file (filename_out, &dwg);
                   }
                 else if (
-#ifdef _WIN32
-      strEQc (filename_out, "NUL")
-#else
-      strEQc (filename_out, "/dev/null")
-#endif
-                         )
+#  ifdef _WIN32
+                    strEQc (filename_out, "NUL")
+#  else
+                    strEQc (filename_out, "/dev/null")
+#  endif
+                )
                   {
                     error = dwg_write_file (filename_out, &dwg);
                   }
@@ -332,11 +412,18 @@ main (int argc, char *argv[])
       if (error)
         fprintf (stderr, "WRITE ERROR 0x%x %s\n", error, filename_out);
 
+#if defined __SANITIZE_ADDRESS__ || __has_feature(address_sanitizer)
+      {
+        char *asanenv = getenv ("ASAN_OPTIONS");
+        if (!asanenv)
+          do_free = 1;
+        // detect_leaks is enabled by default. see if it's turned off
+        else if (strstr (asanenv, "detect_leaks=0") == NULL) /* not found */
+          do_free = 1;
+      }
+#endif
       // forget about leaks. really huge DWG's need endlessly here.
       if (do_free
-#if defined __SANITIZE_ADDRESS__ || __has_feature(address_sanitizer)
-          || 1 /* but skip detect_leaks via __asan_default_options() */
-#endif
 #ifdef HAVE_VALGRIND_VALGRIND_H
           || (RUNNING_ON_VALGRIND)
 #endif
